﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

using System.IO;

using ImageMagick;

namespace HexViewer
{
  public partial class Form1 : Form
  {

    private const bool MAKE_ANIMATION = true;

    private HexMapDrawer hxDrawer;

    static int[,] mapData;

    #region コンストラクタ

    public Form1()
    {
      InitializeComponent();

      //地図読み込み
      using (StreamReader sReader = new StreamReader("map.csv"))
      {
        sReader.ReadLine(); //ヘッダ

        List<string[]> data = new List<string[]>();
        string buff;
        while ((buff = sReader.ReadLine()) != null)
        { data.Add(buff.Split(',')); }

        mapData = new int[data[0].Length - 1, data.Count];
        for (int i = 0; i < data.Count; i++)
          for (int j = 0; j < data[0].Length - 1; j++)
            mapData[j, i] = int.Parse(data[i][j + 1]);
      }

      //地図作成
      hxDrawer = new HexMapDrawer(mapData.GetLength(0), mapData.GetLength(1), 15);
    }

    #endregion

    #region 地図描画

    private void btnDrawMap_Click(object sender, EventArgs e)
    {
      //描画先とするImageオブジェクトを作成する
      Bitmap canvas = new Bitmap(hxDrawer.Width, hxDrawer.Height);
      Graphics gr = Graphics.FromImage(canvas);

      //アンチエイリアス処理
      gr.SmoothingMode = SmoothingMode.AntiAlias;

      //背景を描画
      if (cbxBackGround.Checked)
      {
        Image img = Image.FromFile("backGround.gif");
        gr.DrawImage(img, 0, 0, hxDrawer.Width, hxDrawer.Height);
      }

      //ペン・ブラシ設定
      hxDrawer.PathPen = Pens.Gray;
      for (int i = 0; i < mapData.GetLength(0); i++)
      {
        for (int j = 0; j < mapData.GetLength(1); j++)
        {
          Brush brsh;
          switch (mapData[i, j])
          {
            case 1:
              brsh = new SolidBrush(Color.FromArgb(0, 0, 0, 0));  //標準
              break;
            case 2:
              brsh = new SolidBrush(Color.FromArgb(50, 0, 0, 0));  //通路
              break;
            case 3:
              brsh = new SolidBrush(Color.FromArgb(20, 0, 0, 0));  //席
              break;
            case 4:
              brsh = new SolidBrush(Color.FromArgb(50, 0, 0, 0));  //扉
              break;
            default:  //通行不可
              brsh = new HatchBrush(HatchStyle.WideUpwardDiagonal, Color.Black, Color.FromArgb(50, 255, 255, 255));
              break;
          }
          hxDrawer.SetBrush(i, j, brsh);
        }
      }

      //HEX描画
      hxDrawer.DrawMap(gr);

      //リソース解放
      gr.Dispose();

      //描画
      pictureBox1.Image = canvas;

      //保存
      canvas.Save("backGroundWithHex.png", System.Drawing.Imaging.ImageFormat.Png);
    }

    #endregion

    #region アニメーション

    private void btnAnimate_Click(object sender, EventArgs e)
    {
      Thread thread = new Thread(new ThreadStart(animate));
      thread.Start();
    }

    private void animate()
    {
      //背景を取得
      Image img = Image.FromFile("backGroundWithHex.png");

      //ルート描画
      if (File.Exists("route.csv"))
      {
        using (MagickImageCollection collection = new MagickImageCollection())
        using (StreamReader sReader = new StreamReader("route.csv"))
        {
          string buff;
          string lstLine = "";
          sReader.ReadLine(); //タイトル行

          //すっ飛ばす
          for (int i = 0; i < 0; i++) sReader.ReadLine();

          while ((buff = sReader.ReadLine()) != null)
          {
            if (lstLine != buff)
            {
              lstLine = buff;

              //描画先とするImageオブジェクトを作成する
              Bitmap canvas = new Bitmap(hxDrawer.Width, hxDrawer.Height);
              Graphics gr = Graphics.FromImage(canvas);

              //背景描画
              gr.DrawImage(img, 0, 0, hxDrawer.Width, hxDrawer.Height);

              //在室者描画
              string[] bf = buff.Split(',');
              int ocNum = (bf.Length - 1) / 3;
              for (int i = 0; i < ocNum; i++)
              {
                int ss = 1 + 3 * i;
                int stts = int.Parse(bf[ss]);
                if (stts == 1)
                  hxDrawer.DrawCircle(gr, int.Parse(bf[ss + 1]), int.Parse(bf[ss + 2]), Brushes.Blue, Pens.Black);
                else if (stts == 2)
                  hxDrawer.DrawCircle(gr, int.Parse(bf[ss + 1]), int.Parse(bf[ss + 2]), Brushes.Red, Pens.Black);
                else if (stts == 3)
                  hxDrawer.DrawCircle(gr, int.Parse(bf[ss + 1]), int.Parse(bf[ss + 2]), Brushes.Pink, Pens.Black);
                else if (stts == 4)
                  hxDrawer.DrawCircle(gr, int.Parse(bf[ss + 1]), int.Parse(bf[ss + 2]), Brushes.Green, Pens.Black);
              }

              //リソース解放
              gr.Dispose();

              //GIFアニメ
              if (MAKE_ANIMATION)
              {
                MagickImage mImg = new MagickImage(canvas);
                mImg.AnimationDelay = 10;
                collection.Add(mImg);
              }

              //PictureBoxに描画
              pictureBox1.Invoke(new Action(() =>
              {
                pictureBox1.Image = canvas;
              }));
              Thread.Sleep(50);
            }
          }

          if (MAKE_ANIMATION)
          {
            collection.Optimize();
            collection.Write("test.gif");
          }
        }
      }

      MessageBox.Show("End Animation");
    }

    #endregion

    #region 存在率

    private void btnOccupied_Click(object sender, EventArgs e)
    {
      //存在率ファイルを読み込む
      bool[,] zero = new bool[hxDrawer.XNumber, hxDrawer.YNumber];
      double[,] rate = new double[hxDrawer.XNumber, hxDrawer.YNumber];
      double max = double.MinValue;
      double min = double.MaxValue;
      using (StreamReader sReader = new StreamReader("occupiedRate.csv"))
      {
        sReader.ReadLine(); //タイトル        
        for (int i = 0; i < hxDrawer.YNumber; i++)
        {
          string[] bf = sReader.ReadLine().Split(',');
          for (int j = 0; j < hxDrawer.XNumber; j++)
          {
            double val = double.Parse(bf[j + 1]);
            zero[j, i] = (val == 0);
            if (!zero[j, i])
            {
              rate[j, i] = Math.Log(val);
              max = Math.Max(max, rate[j, i]);
              min = Math.Min(min, rate[j, i]);
            }
          }          
        }
      }
      //0-255に正規化
      double dtl = 255 / (max - min);
      for (int i = 0; i < rate.GetLength(0); i++)
        for (int j = 0; j < rate.GetLength(1); j++)
          rate[i, j] = (rate[i, j] - min) * dtl;

      //描画先とするImageオブジェクトを作成する
      Bitmap canvas = new Bitmap(hxDrawer.Width, hxDrawer.Height);
      Graphics gr = Graphics.FromImage(canvas);

      //アンチエイリアス処理
      gr.SmoothingMode = SmoothingMode.AntiAlias;

      //背景を描画
      if (cbxBackGround.Checked)
      {
        Image img = Image.FromFile("backGround.gif");
        gr.DrawImage(img, 0, 0, hxDrawer.Width, hxDrawer.Height);
      }

      //ペン・ブラシ設定
      hxDrawer.PathPen = Pens.Gray;
      //hxDrawer.PathPen = Pens.Transparent;
      for (int i = 0; i < mapData.GetLength(0); i++)
      {
        for (int j = 0; j < mapData.GetLength(1); j++)
        {
          Brush brsh;
          int blk = (int)rate[i, j];
          switch (mapData[i, j])
          {
            case 1:
            case 2:
            case 3:
            case 4:
              if(zero[i,j]) brsh = new SolidBrush(Color.FromArgb(0, 0, 0, 0));  //標準
              else brsh = new SolidBrush(Color.FromArgb(blk, 0, 0, 0));  //標準
              break;
            default:  //通行不可
              brsh = new HatchBrush(HatchStyle.WideUpwardDiagonal, Color.Black, Color.FromArgb(50, 255, 255, 255));
              break;
          }
          hxDrawer.SetBrush(i, j, brsh);
        }
      }

      //HEX描画
      hxDrawer.DrawMap(gr);

      //リソース解放
      gr.Dispose();

      //描画
      pictureBox1.Image = canvas;

      //保存
      canvas.Save("occupantRate.png", System.Drawing.Imaging.ImageFormat.Png);
    }

    #endregion

    #region その他

    private void pictureBox1_MouseClick(object sender, MouseEventArgs e)
    {
      int xPos, yPos;
      if (hxDrawer.GetHexPosition(e.Location, out xPos, out yPos))
        MessageBox.Show((xPos + 1) + "," + (yPos + 1));
    }


    #endregion

  }
}
