﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WorkingDead
{
  /// <summary>人感センサ</summary>
  public class HumanSensor
  {

    #region インスタンス変数

    /// <summary>地図</summary>
    private AStarMap myMap;

    /// <summary>検知の起点（X座標）を取得する</summary>
    public int X { get; private set; }

    /// <summary>検知の起点（Y座標）を取得する</summary>
    public int Y { get; private set; }

    /// <summary>検知の左右幅を取得する</summary>
    public int Width { get; private set; }

    /// <summary>検知の上下幅を取得する</summary>
    public int Height { get; private set; }

    /// <summary>照明台数を設定・取得する</summary>
    public int LampNumber { get; set; }

    #endregion

    #region コンストラクタ

    /// <summary>インスタンスを初期化する</summary>
    /// <param name="aMap">地図</param>
    /// <param name="x">検知の起点（X座標）</param>
    /// <param name="y">検知の起点（Y座標）</param>
    /// <param name="width">検知の左右幅</param>
    /// <param name="height">検知の上下幅</param>
    public HumanSensor(AStarMap aMap, int x, int y, int width, int height)
    {
      this.myMap = aMap;
      this.X = x;
      this.Y = y;
      this.Width = width;
      this.Height = height;
    }

    #endregion

    #region インスタンスメソッド

    /// <summary>人間が存在するか</summary>
    /// <returns>存在する場合にはtrue</returns>
    public bool FindOccupant()
    {
      for (int i = 0; i < Width; i++)
        for (int j = 0; j < Height; j++)
          if (myMap.GetTile(X + i, Y + j).Agent != null) return true;

      return false;
    }

    #endregion

  }
}
