﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.IO;

/// <summary>働きアリ達が事務室をさまよい歩く</summary>
namespace WorkingDead
{
  class Program
  {

    #region 定数宣言

    /// <summary>乱数生成シード</summary>
    private const int RND_SEED = 2;

    /// <summary>反復計算回数</summary>
    private const int ITER_NUM = 10;

    /// <summary>1回の計算時間数[min]</summary>
    private const int MINUTES = 10 * 60;

    /// <summary>計算時間間隔[sec]</summary>
    private const double DELTA_T = 0.4;

    /// <summary>計算時間間隔[milli sec]</summary>
    private const int MILLSEC = (int)(DELTA_T * 1000);

    #endregion

    #region メイン処理

    static void Main(string[] args)
    {
      //引数設定
      double occRate, minLigtRate;
      int fadeTime, waitTime, meetNum;
      if (args.Length == 5)
      {
        minLigtRate = double.Parse(args[0]);
        fadeTime = int.Parse(args[1]) * 60;
        waitTime = int.Parse(args[2]) * 60;
        meetNum = int.Parse(args[3]);
        occRate = double.Parse(args[4]);
      }
      else
      {
        minLigtRate = 0.05;
        fadeTime = 3 * 60;
        waitTime = 5 * 30;
        meetNum = 5;
        occRate = 1.0;
      }

      Random rnd = new Random(RND_SEED);

      //地図読み込み
      int[,] mapData;
      using (StreamReader sReader = new StreamReader("map.csv"))
      {
        sReader.ReadLine(); //ヘッダ

        List<string[]> data = new List<string[]>();
        string buff;
        while ((buff = sReader.ReadLine()) != null)
        { data.Add(buff.Split(',')); }

        mapData = new int[data[0].Length - 1, data.Count];
        for (int i = 0; i < data.Count; i++)
          for (int j = 0; j < data[0].Length - 1; j++)
            mapData[j, i] = int.Parse(data[i][j + 1]);
      }

      //計算実行+位置情報書き出し
      int[][,] ocNum = new int[ITER_NUM][,];
      double[][,] ltRate = new double[ITER_NUM][,];
      double[] ltRateSum = new double[ITER_NUM];
      int[] rndSeeds = new int[ITER_NUM];
      for (int i = 0; i < ITER_NUM; i++) rndSeeds[i] = rnd.Next();
      Parallel.For(0, ITER_NUM, i =>
      {
        if (i == 0) runAStar(i, rndSeeds[i], mapData, occRate, meetNum, waitTime, fadeTime, minLigtRate, true, out ocNum[i], out ltRate[i], out ltRateSum[i]);
        else runAStar(i, rndSeeds[i], mapData, occRate, meetNum, waitTime, fadeTime, minLigtRate, false, out ocNum[i], out ltRate[i], out ltRateSum[i]);
        //runAStar(i, rndSeeds[i], mapData, occRate, meetNum, waitTime, fadeTime, minLigtRate, false, out ocNum[i], out ltRate[i], out ltRateSum[i]);
      });
      //在室人数積算
      for (int i = 0; i < ocNum[0].GetLength(0); i++)
        for (int j = 0; j < ocNum[0].GetLength(1); j++)
          for (int k = 1; k < ocNum.Length; k++)
            ocNum[0][i, j] += ocNum[k][i, j];
      //点灯率積算
      for (int i = 0; i < ltRate[0].GetLength(0); i++)
        for (int j = 0; j < ltRate[0].GetLength(1); j++)
          for (int k = 1; k < ltRate.Length; k++)
            ltRate[0][i, j] += ltRate[k][i, j];

      //存在率[-]
      using (StreamWriter sWriter = new StreamWriter("occupiedRate.csv"))
      {
        for (int j = 0; j < ocNum[0].GetLength(0); j++)
          sWriter.Write("," + (j + 1));
        sWriter.WriteLine();

        for (int i = 0; i < ocNum[0].GetLength(1); i++)
        {
          sWriter.Write((i + 1));
          double bf = DELTA_T / (ITER_NUM * MINUTES * 60);
          for (int j = 0; j < ocNum[0].GetLength(0); j++)
            sWriter.Write("," + (ocNum[0][j, i] * bf));
          sWriter.WriteLine();
        }
      }

      //点灯率[-]
      using (StreamWriter sWriter = new StreamWriter("lightingRate.csv"))
      {
        double bf = DELTA_T / (60 * MINUTES * ITER_NUM);
        for (int i = 0; i < ltRate[0].GetLength(1); i++)
        {
          for (int j = 0; j < ltRate[0].GetLength(0); j++)
          {
            if (j == 0) sWriter.Write((ltRate[0][j, i] * bf).ToString("F4"));
            else sWriter.Write("," + (ltRate[0][j, i] * bf).ToString("F4"));
          }
          sWriter.WriteLine();
        }
      }

      //点灯率[-]
      using (StreamWriter sWriter = new StreamWriter("lightingRate_Sum.txt", true))
      {
        double sum = 0;
        for (int i = 0; i < ltRateSum.Length; i++)
          sum += ltRateSum[i];
        sum *= DELTA_T / (60 * MINUTES * ITER_NUM) / 88;
        sWriter.WriteLine(sum);
      }

    }

    #endregion

    #region 移動処理

    private static void runAStar
      (int threadNumber, int rndSeed, int[,] mapData, double occupancyRate, int meetingNumber,
      int waitingTime, int fadeoutTime, double minLightingRate, bool output, out int[,] ocNum, out double[,] ltRate, out double ltRateSum)
    {
      //地図作成
      AStarMap aMap = new AStarMap((uint)mapData.GetLength(0), (uint)mapData.GetLength(1));
      aMap.SetTileType(mapData);
      aMap.Now = new DateTime(1999, 1, 1, 0, 0, 0);

      //照明制御システム
      LightingController lCtrl = new LightingController(aMap, LightingController.Grid.grd64, 0.00); //ここ変える
      lCtrl.WaitingTime = waitingTime;
      lCtrl.FadeoutTime = fadeoutTime;
      lCtrl.MinimumLightingRate = minLightingRate;
      ltRate = new double[lCtrl.XNumber, lCtrl.YNumber];
      ltRateSum = 0;

      //執務者作成
      OfficeOccupant[] ocpnts = makeOccupants(aMap, rndSeed, occupancyRate, meetingNumber);
      for (int i = 0; i < ocpnts.Length; i++)
      {
        ocpnts[i].SetPosition(ocpnts[i].SeatTile.X, ocpnts[i].SeatTile.Y);  //自席に着席
        ocpnts[i].MakeSchedule();
      }

      //着席テスト//コメントアウトせよ
      for (int i = 0; i < 50; i++) ocpnts[i].SetPosition(i + 20, 5);
      for (int i = 51; i < ocpnts.Length; i++) ocpnts[i].SetPosition(i - 51 + 20, 7);

      //ドアの場所を設定
      for (int i = 0; i < mapData.GetLength(0); i++)
        for (int j = 0; j < mapData.GetLength(1); j++)
          if (mapData[i, j] == 4)
          {
            for (int k = 0; k < ocpnts.Length; k++)
              ocpnts[k].Doors.Add(aMap.GetTile(i, j));
          }

      ocNum = new int[aMap.XTiles, aMap.YTiles];

      StreamWriter sWriter1 = null;
      StreamWriter sWriter2 = null;
      if (output)
      {
        sWriter1 = new StreamWriter("route.csv");
        sWriter2 = new StreamWriter("lighting.csv");
      }

      DateTime endTime = aMap.Now.AddMinutes(MINUTES);

      if (output)
      {
        //タイトル行:人の動き
        sWriter1.Write("DateTime");
        for (int i = 0; i < ocpnts.Length; i++)
          sWriter1.Write(", Occupant-" + (i + 1) + " Status, Occupant-" + (i + 1) + " X, Occupant-" + (i + 1) + " Y");
        sWriter1.WriteLine();

        //タイトル行:照明出力
        sWriter2.Write("DateTime");
        for (int i = 0; i < lCtrl.YNumber; i++)
          for (int j = 0; j < lCtrl.XNumber; j++)
            sWriter2.Write(", LightingZone-" + (j + 1) + "-" + (i + 1));
        sWriter2.WriteLine();
      }

      while (aMap.Now < endTime)
      {
        if (aMap.Now.Minute % 10 == 0 && aMap.Now.Second == 0 && aMap.Now.Millisecond == 0)
          Console.WriteLine("Thread " + threadNumber + ": " + aMap.Now.ToShortTimeString());

        if (output)
        {
          sWriter1.Write(aMap.Now.ToString());
          sWriter2.Write(aMap.Now.ToString());
        }

        //執務者移動
        aMap.MoveAgents(DELTA_T);

        //照明制御更新
        lCtrl.Update();
        for (int i = 0; i < lCtrl.XNumber; i++)
          for (int j = 0; j < lCtrl.YNumber; j++)
            ltRate[i, j] += lCtrl.GetPower(i, j);
        ltRateSum += lCtrl.GetPower();

        //執務者行動判定
        for (int i = 0; i < ocpnts.Length; i++)
        {
          ocpnts[i].Update();
          //外出中または自席はカウントしない
          if ((ocpnts[i].Status != OfficeOccupant.State.Outgoing) && (!ocpnts[i].CurrentTile.Equals(ocpnts[i].SeatTile)))
            ocNum[ocpnts[i].CurrentTile.X, ocpnts[i].CurrentTile.Y]++;
        }

        //状態書き出し
        if (output)
        {
          //人の動き
          for (int i = 0; i < ocpnts.Length; i++)
          {
            int stts = 0; //Outgoing
            if (ocpnts[i].Status == OfficeOccupant.State.Meeting || ocpnts[i].IsLocked) stts = 1;
            else if (ocpnts[i].Status == OfficeOccupant.State.None) stts = 2;
            else if (ocpnts[i].Status == OfficeOccupant.State.StartOutgoing) stts = 3;
            else if (ocpnts[i].Status == OfficeOccupant.State.StartMeeting) stts = 4;

            sWriter1.Write(", " + stts + ", " + ocpnts[i].CurrentTile.X + ", " + ocpnts[i].CurrentTile.Y);
          }
          sWriter1.WriteLine();

          //照明状態
          for (int i = 0; i < lCtrl.YNumber; i++)
            for (int j = 0; j < lCtrl.XNumber; j++)
              sWriter2.Write(", " + lCtrl.GetPower(j, i).ToString("F2"));
          sWriter2.WriteLine();
        }
      }

      if (output)
      {
        sWriter1.Close();
        sWriter2.Close();
      }
    }

    #endregion

    #region 執務者作成処理

    private static OfficeOccupant[] makeOccupants(AStarMap map, int randSeed, double occupancyRate, int meetingNumber)
    {
      Random rnd = new Random(randSeed);
      List<OfficeOccupant> ocpnts = new List<OfficeOccupant>();

      int[] lInd = new int[] { 8, 17, 26, 35, 45, 54, 64, 73, 82, 92 };
      int[] rInd = new int[] { 12, 21, 30, 40, 49, 59, 68, 77, 87, 96 };
      for (int i = 0; i < 10; i++)
      {
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(lInd[i], 10), AStarMap.Direction.LeftUp, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(lInd[i], 13), AStarMap.Direction.LeftUp, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(lInd[i], 20), AStarMap.Direction.LeftUp, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(lInd[i], 23), AStarMap.Direction.LeftUp, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(lInd[i], 26), AStarMap.Direction.LeftUp, rnd.Next()));

        ocpnts.Add(new OfficeOccupant(map, map.GetTile(rInd[i], 9), AStarMap.Direction.RightDown, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(rInd[i], 12), AStarMap.Direction.RightDown, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(rInd[i], 19), AStarMap.Direction.RightDown, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(rInd[i], 22), AStarMap.Direction.RightDown, rnd.Next()));
        ocpnts.Add(new OfficeOccupant(map, map.GetTile(rInd[i], 25), AStarMap.Direction.RightDown, rnd.Next()));
      }

      ocpnts.Add(new OfficeOccupant(map, map.GetTile(8, 29), AStarMap.Direction.LeftUp, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(14, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(23, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(32, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(42, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(51, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(61, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(70, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(79, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(89, 29), AStarMap.Direction.Down, rnd.Next()));
      ocpnts.Add(new OfficeOccupant(map, map.GetTile(96, 28), AStarMap.Direction.RightDown, rnd.Next()));

      //人員密度に合うように間引く
      int rmvNum = (int)(ocpnts.Count * (1.0 - occupancyRate));
      for (int i = 0; i < rmvNum; i++)
      {
        OfficeOccupant oc = ocpnts[rnd.Next(ocpnts.Count)];
        ocpnts.Remove(oc);
        map.RemoveAgent(oc);
      }

      for (int i = 0; i < ocpnts.Count; i++) ocpnts[i].MeetingNumber = meetingNumber;

      return ocpnts.ToArray();
    }

    #endregion

  }
}
